//RadioProbe
//Copyright © 2000-2001, Paul Yu

#include "RadioProbeLooper.h"
#include "RadioProbeConstants.h"
#include <String.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <netinet/in.h>
#include <NetKit.h>
#include <NetworkKit.h>
#include <Debug.h>
#include <File.h>
#include <Path.h>
#include <FindDirectory.h>
#include <Entry.h>
#include <Roster.h>
#include <Handler.h>

RadioProbeLooper::~RadioProbeLooper()
{
	/*	*/
}

void RadioProbeLooper::MessageReceived(BMessage *msg)
{
	BString artist;
	BString title;
	status_t err;
	BHandler *view;
	const char *server;
	uint16 port;
	bool enable;

	switch(msg->what)
	{
		case P_LAUNCH_PLAYLIST:
		{
			LaunchPlayList();
		}
		break;

		case P_UPDATE_CURRENT_SONG:
		{
			if (msg->FindPointer("view", (void **)&view) < B_OK)
				return;
		
			if ((err = GetCurrentSong(title, artist)) < B_OK)
			{
				title.SetTo(NO_CONNEXION);
				artist.SetTo("(");
				artist.Append(strerror(err));
				artist.Append(")");
			}
			
			BMessage info(P_UPDATE_CURRENT_SONG);
			BMessenger messenger(view);
			if (info.AddString("title", title.String()) == B_OK &&
				info.AddString("artist", artist.String()) == B_OK)
				messenger.SendMessage(&info);
		}
		break;
	
		case P_SET_SERVER:
		{
			if ((msg->FindBool("enable", &enable) == B_OK) &&
				enable &&
				(msg->FindString("server", &server) == B_OK) &&
				(msg->FindInt16("port", (int16 *)&port) == B_OK))
			{
				mServer.SetTo(server);
				mPort = port;
			}
			else
			{
				mServer.SetTo(kHTTPServer);
				mPort = 80;
			}
		}
		break;

		case P_SET_STREAM:
		{
			const char *stream;
			if (msg->FindString("stream", &stream) == B_OK)
				mStream.SetTo(stream);
		}
		break;

		default:
			BLooper::MessageReceived(msg);
	}
}

status_t RadioProbeLooper::GetCurrentSong(BString& title, BString& artist)
{
	const char *request = "GET " kSongURL " HTTP/1.0\r\n" "User-Agent: " kUserAgent "\r\n\r\n";
	const int32 BUFFER_SIZE = 4096;
	BNetEndpoint http;
	BString page;
	char *buffer;
	status_t err;
	int32 begin, end, len;
	bool found = false;

	try
	{
		buffer = new char[BUFFER_SIZE];
	}
	catch(...)
	{
		return B_NO_MEMORY;
	}
	
	http.SetTimeout(10000000);
	if (((err = http.Connect(mServer.String(), mPort)) < B_OK) ||
		((err = http.Send(request, strlen(request))) < B_OK))
		goto ReadError;

	//Skip the HTTP header (assume we got "200 OK")
	ReadMore:
	if ((err = http.Receive(buffer, BUFFER_SIZE, 0)) > 0)
	{
		char *p = buffer;
		char *max = buffer+err-3;
	
		while(p < max)
		{
			if (*p == '\r' && *(p+1) == '\n')
			{
				p += 2;
				if (*p == '\r' && *(p+1) == '\n')
				{
					found = true;
					break;	
				}
			}
			else if (*p == '\n' && *(p+1) == '\n')
			{
				found = true;
				break;
			}
			++p;
		}
		
#if DEBUG
		*(p + 1) = '\0';
		printf("%s", buffer);
#endif
		
		if (found)
		{
			p+=2;
			
			if (err-(p-buffer) > 0)
			{
				try
				{
					page.Append(p, err-(p-buffer));
				}
				catch(...)
				{
					err = B_NO_MEMORY;
					goto ReadError;
				}
			}
		}
		else
			goto ReadMore;
	}
	else
	{
		if (err == B_OK)
			err = B_FILE_NOT_FOUND;
		goto ReadError;
	}

	//Receive the song info
	while((err = http.Receive(buffer, BUFFER_SIZE, 0)) > 0)
	{
		try
		{
			page.Append(buffer, err);
		}
		catch(...)
		{
			err = B_NO_MEMORY;
			goto ReadError;
		}
	}
	http.Close();
	
	if (err < 0)
		goto ReadError;
	
	len = page.Length();
	if (err == 0 && len == 0)
	{
		err = B_TIMED_OUT;
		goto ReadError;
	}

#if DEBUG
	printf("\n%s", page.String());
#endif

	//It's okay not to have a current song

#ifdef BeOSRADIO
	if ((len == 0) || 
		((begin = page.FindFirst(";")) < 0) ||
		((end = page.FindFirst("\n")) <= begin+1))
	{
		artist.SetTo("(none)");
		title.SetTo("(none)");
	}
	else
	{
		const char *s = page.String();
		artist.SetTo(s, begin);
		title.SetTo(s+begin+1, end-begin-1);
	}
#endif

#ifdef AngryRadio
	if ((len == 0) || 
		((begin = page.FindFirst(";")) < 0) ||
		((end = page.FindFirst("\n")) <= begin+1))
	{
		artist.SetTo("(none)");
		title.SetTo("(none)");
	}
	else
	{
		const char *s = page.String();
		artist.SetTo(s, begin);
		title.SetTo(s+begin+1, end-begin-1);
		artist.RemoveFirst("   ");
	}
#endif

#ifdef Technospider
	if ((len == 0) || 
		((begin = page.FindFirst("Song:")) < 0) ||
		((end = page.FindFirst("\n", begin)) <= begin+5))
	{
		artist.SetTo("(none)");
		title.SetTo("(none)");
	}
	else
	{
		const char *s = page.String();
		title.SetTo(s+begin+5, end-begin-5);
		
		if (((begin = page.FindFirst("Artist:")) < 0) ||
			((end = page.FindFirst("\n", begin)) <= begin+7))
			artist.SetTo("(none)");
		else
			artist.SetTo(s+begin+7, end-begin-7);
			
		title.ReplaceAll("+", " ");
		artist.ReplaceAll("+", " ");
	}
#endif

#ifdef Electronic_Deconstruction
	if ((len == 0) || 
		((begin = page.FindFirst("<!-- starttag -->")) < 0) ||
		((end = page.FindFirst(" - ", begin + 17)) <= begin+17))
	{
		artist.SetTo("(none)");
		title.SetTo("(none)");
	}
	else
	{
		const char *s = page.String();

		artist.SetTo(s+begin+17, end-begin-17);
		
		begin = end + 3;
		
		if (((end = page.FindFirst("<!-- stoptag -->", begin)) <= begin))
			title.SetTo("(none)");
		else
			title.SetTo(s+begin, end-begin);
	}
#endif

	printf("\nThe current song is <%s> by <%s>.\n", title.String(), artist.String());
	
	ReadError:
		http.Close();
		delete [] buffer;
		return err;
}

bool RadioProbeLooper::Delete(void *object)
{
	BString *s = (BString *)object;
	delete s;
	return false;
}

status_t RadioProbeLooper::LaunchPlayList()
{
	const char *request_tail = " HTTP/1.0\r\n"	"User-Agent: " kUserAgent "\r\n" "\r\n";	
	BString request;
	BFile file;
	BPath path;
	BNetEndpoint http;
	status_t err;
	char *buffer = NULL;
	bool found = false;
	const int32 BUFFER_SIZE = 4096;
	entry_ref ref;
	BEntry entry;

	if (((err = find_directory(B_COMMON_TEMP_DIRECTORY, &path)) < B_OK) ||
		((err = path.Append(kListenFile)) < B_OK))
		return err;
	
	try
	{
		request.SetTo("GET ");
		if (mStream.Length() == 0)
			mStream.SetTo(kLowURL);
		request.Append(mStream.String());
		request.Append(request_tail);
		buffer = new char[BUFFER_SIZE];
	}
	catch(...)
	{
		return B_NO_MEMORY;	
	}

	//BNetDebug::Enable(true);

	http.SetTimeout(10000000);
	
	if (((err = file.SetTo(path.Path(), B_WRITE_ONLY | B_ERASE_FILE | B_CREATE_FILE)) < B_OK) ||
		((err = http.Connect(mServer.String(), mPort)) < B_OK) ||
		((err = http.Send(request.String(), request.Length())) < B_OK))
		goto ReadError;

	ReadMore:

	if ((err = http.Receive(buffer, BUFFER_SIZE, 0)) > 0)
	{
		char *p = buffer;
		char *max = buffer+err-3;
	
		while(p < max)
		{
			if (*p == '\r' && *(p+1) == '\n')
			{
				p += 2;
				if (*p == '\r' && *(p+1) == '\n')
				{
					found = true;
					break;	
				}
			}
			else if (*p == '\n' && *(p+1) == '\n')
			{
				found = true;
				break;
			}
			++p;
		}
		
#if DEBUG
		*(p + 1) = '\0';
		printf("%s", buffer);
#endif
		
		if (found)
		{
			p+=2;
			
			if (err-(p-buffer) > 0)
				if ((err = file.Write(p, err-(p-buffer))) < B_OK)
					goto ReadError;
		}
		else
			goto ReadMore;
	}
	else
	{
		if (err == B_OK)
			err = B_FILE_NOT_FOUND;
		goto ReadError;
	}

	while ((err = http.Receive(buffer, BUFFER_SIZE, 0)) > 0)
	{
		if ((err = file.Write(buffer, err)) < B_OK)
			break;
	}

	file.Unset();

	if ((err = be_roster->FindApp(kListenFileType, &ref)) == B_OK)
	{
		const char *argv[3];
		argv[0] = ref.name;
		argv[1] = path.Path();
		argv[2] = NULL;
		be_roster->Launch(kListenFileType, 2, (char **)argv);
	}
	
	ReadError:
		//printf("%s\n", strerror(err));

		http.Close();
		file.Unset();
		delete [] buffer;
		if (err < B_OK)
			unlink(path.Path());
		return err;
}
