//RadioProbe
//Copyright © 2000-2001, Paul Yu

#include "RadioProbeWindow.h"
#include "RadioProbeView.h"
#include "RadioProbeConstants.h"
#include <Application.h>
#include <File.h>
#include <Path.h>
#include <FindDirectory.h>
#include <stdio.h>

RadioProbeWindow::RadioProbeWindow()
	: BWindow(BRect(100,80,360,152), kWindowName, B_TITLED_WINDOW,
	B_WILL_ACCEPT_FIRST_CLICK | B_NOT_ZOOMABLE | B_ASYNCHRONOUS_CONTROLS)
{
	BFont font(be_plain_font);
	font.SetSize(10);

	float width = font.StringWidth(LISTEN kListenLink)+12;
	SetSizeLimits(width, 1610, 48, 72);

	mView = new RadioProbeView(Bounds());
	AddChild(mView);

	BRect frame;
	BView *menu = FindView("menu");
	frame = menu->Frame();
	width = (frame.right - frame.left) + width + 5;
	ResizeTo(width, 72);
}

RadioProbeWindow::RadioProbeWindow(BMessage *archive)
	: BWindow(archive)
{
	BMessage views;
	BFont font(be_plain_font);
	font.SetSize(10);

	//pre 1.02 compatibility
	if (archive->FindMessage("_views", &views) < B_OK)
	{
		mView = new RadioProbeView(Bounds());
		AddChild(mView);
	}
	else
		mView = FindView(kReplicantName);

	//pre 0.25 compatibility
	SetFlags(B_WILL_ACCEPT_FIRST_CLICK | B_NOT_ZOOMABLE | B_ASYNCHRONOUS_CONTROLS);
	
	//adjust for system font preferences
	SetSizeLimits(font.StringWidth(LISTEN kListenLink)+12, 1610, 48, 72);
}

void RadioProbeWindow::WorkspacesChanged(uint32 oldWorkspaces, uint32 newWorkspaces)
{
	if (Lock())
	{
		mView->Invalidate();
		Unlock();	
	}
}

bool RadioProbeWindow::QuitRequested()
{
	BPath path;
	BFile file;
	BMessage archive;
	status_t err;

	if (((err = find_directory(B_USER_SETTINGS_DIRECTORY, &path)) < B_OK) ||
		((err = path.Append(kRadioProbePrefsFile)) < B_OK) ||
		((err = Archive(&archive, true)) < B_OK) ||
		((err = file.SetTo(path.Path(), B_WRITE_ONLY | B_ERASE_FILE | B_CREATE_FILE)) < B_OK) ||
		((err = archive.Flatten(&file)) < B_OK))
	{
		fprintf(stderr, IMPOSSIBLE_SAVE_PREFS " (%s)\n", strerror(err));
	}
	file.Unset();

	//Hide();

	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}
